SYD-OCI(1)

# NAME

syd-oci - OCI container runtime

# SYNOPSIS

*syd-oci* _[OPTIONS]_ _[COMMAND]_

# DESCRIPTION

*syd-oci* is an OCI container runtime implementation for Syd.

All common subcommands are supported: create, start, state, kill,
delete, pause, resume, exec, run, list, ps, spec, events, features,
update and checkpoint.

# INTEGRATION

*syd-oci* is a thin wrapper that integrates the Syd sandbox into
containers. It is largely based on *youki* and is compatible with
Docker, Podman and CRI-O. To get *syd-oci*, you should build Syd with
the "oci" feature. To use *syd-oci* with Docker you have two options:

. Start dockerd manually with --add-runtime="syd=/bin/syd-oci", then do \
for example "docker run -it --runtime syd alpine" when starting containers.
. Add the following snippet to your "/etc/docker/daemon.json" file:

```
{
"runtimes": { "syd": { "path": "/bin/syd-oci" } },
"default-runtime": "syd"
}
```

You may need to adapt the path to *syd-oci* depending on your installation.

To use with Podman is similar, just pass "--runtime=/bin/syd-oci" as an option to podman run.

# CONFIGURATION

The configuration directory of *syd-oci* is one of the following:

- For system-wide containers: "/etc/syd/oci"
- For rootless containers, one of the following:
	- "${XDG_CONFIG_HOME}/syd/oci" where XDG_CONFIG_HOME is usually ~/.config
	- "${HOME}/.syd/oci" if XDG_CONFIG_HOME is not set.

*syd-oci* attempts to configure the Syd sandbox in the following order,
and parses the first file or profile it locates and stops processing:

. If hostname and domainname is defined for the container, try to load
  "${SYD_CONFIG_DIR}/${hostname}.${domainname}.syd-3".
. If domainname is defined for the container, try to load
  "${SYD_CONFIG_DIR}/${domainname}.syd-3".
. If hostname is defined for the container, try to load
  "${SYD_CONFIG_DIR}/${hostname}.syd-3".
. Try to load "${SYD_CONFIG_DIR}/default.syd-3"
. Load the "oci" profile. See "syd-cat oci" for the list of rules.

SYD_CONFIG_DIR in the items above refer to the configuration directory.
Note, the "oci" profile is a fairly restricted profile so you are
strongly recommended to at least create your own configuration defaults
with the default.syd-3 file. Refer to _syd_(5) for the syntax of Syd
configuration files and _syd_(2) for a list of configuration items Syd
understands. A _vim_(1) syntax highlighting file is also provided to
easily edit Syd configuration files. Use "syd-cat file.syd-3" to check a
Syd configuration file for syntax errors.

Finally, note that the "include" directives in the configuration files
are searched within the container image. This allows you to provide
additional image-based sandbox configuration. One possible use could be
to store cryptographic checksums of all executables and their dependent
dynamic libraries in an include file in the image and then use this with
Force Sandboxing for binary verification, see _syd_(7) for more
information on Force Sandboxing.

# SEE ALSO

_syd_(1), _syd_(2), _syd_(5), _syd_(7)

- *syd* homepage: https://sydbox.exherbolinux.org/
- *youki* homepage: https://containers.github.io/youki/

# AUTHORS

Maintained by Ali Polatel. Up-to-date sources can be found at
https://gitlab.exherbo.org/sydbox/sydbox.git and bugs/patches can be
submitted to https://gitlab.exherbo.org/groups/sydbox/-/issues. Discuss
in #sydbox on Libera Chat.
