SYD(1)

# NAME

syd - seccomp and landlock based application sandbox with support for namespaces

# SYNOPSIS

*syd* [-acefhlmpqxEPV] [--] {command [arg...]}

*syd* [-acefhlmpqxEPV] [--] {library.so}

*syd* --sh

# DESCRIPTION

Syd is a utility leveraging the _seccomp_(2) system call for sandboxing
processes on Linux systems version 5.19 or later. It enables fine-grained control
over a process's filesystem and network access _without requiring root
privileges_. Syd is designed for ease of use across a wide array of
architectures, including *x86*, *x86_64*, *x32*, *ARM*, *AArch64*, *MIPS*, and
*MIPS64*, embodying the principle of providing simple, flexible, and robust
access control to Linux users.

The core functionality of Syd revolves around restricting a process's
resource access through several mechanisms:

- *Bind Mounts*: Utilized within a mount namespace to enforce restrictions at \
the *Virtual File System (VFS)* level, such as *read-only*, *nodev*, *noexec*, \
and *nosuid*.
- *Landlock*: Employs read-only and read-write path restrictions at the kernel level.
- *seccomp-bpf*: Applies Secure Computing user filters for kernel-space sandboxing.
- *seccomp-notify*: Enables sandboxing in kernel space with user space \
fallback for dereferencing pointer arguments in system calls, \
including pathnames and network addresses. Access checks utilize \
UNIX shell-style patterns and CIDR notation, defaulting to denying \
system calls with *EACCES* while attempting to emulate successful \
calls to mitigate *Time-of-Check to Time-of-Use (TOCTOU)* \
vulnerabilities. See the *BUGS* section of _syd_(7) manual page for \
cases where system call emulation is currently not possible.

Prerequisites for Syd include a Linux kernel supporting _pidfd_getfd_(2) and
_pidfd_send_signal_(2) system calls, *SECCOMP_USER_NOTIF_FLAG_CONTINUE*
operation in the Secure Computing facility, and preferably the
*CONFIG_CROSS_MEMORY_ATTACH* kernel option. For syscall emulation, Syd uses the
seccomp operation *SECCOMP_IOCTL_NOTIF_ADDFD*. Moreover Syd sets the
*SECCOMP_FILTER_FLAG_WAIT_KILLABLE_RECV* flag to correctly handle interrupts
during tracing. While _Linux version 5.19 or later is required_, for Landlock
support Syd requires a kernel configured with the option *CONFIG_LSM_LANDLOCK*
supporting _Landlock ABI version 3_, with _syd-lock_(1) available as a helper
program to verify kernel support.

Syd is committed to maintaining rigorous security standards by strictly
delimiting the resource space accessible to sandboxed processes. In the
*SECURITY* section of the _syd_(7) manual page, a detailed enumeration of the
security hardening measures implemented by Syd is provided, along with optional
configurations to relax certain restrictions. This flexibility allows for the
accommodation of a diverse range of processes within the sandbox environment.

The approach to security within Syd is methodically designed to balance
robust protection with operational flexibility, ensuring that users have the
ability to fine-tune the sandboxing mechanisms to meet specific requirements. By
offering insights into the hardening techniques and customization options,
Syd empowers users to navigate the trade-offs between security and
functionality effectively.

# OPTIONS

The following options are understood:

|[ *-h*
:< Show usage and exit.
|[ *-V*
:< Show version and exit.
|[ *-c*
:< Login shell compatibility
|[
:< Causes command to be executed under a shell with the *user* profile.
|[
:< The shell to execute is */bin/sh* by default.
|[
:< Use the environment variable *SYD_SH* to override.
|[ *-f*
:< Login shell compatibility.
|[
:< Causes Syd to parse the *user* profile on startup.
|[ *-l*, *--login*
:< Login shell compatibility
|[
:< Causes Syd to parse the *user* profile on startup.
|[ *-q*
:< Enable quick boot mode for faster startup times.
|[
:< This must be passed as the first option or it will be ignored.
|[
:< See the explanation of the environment variable *SYD_QUICK_BOOT* for the safety of this option.
|[ *-x*
:< Enable *trace* aka "dry run" mode.
|[
:< In this mode Syd will allow system calls even if they raise access violations.
|[
:< This mode with extended logging can be used to build sandboxing profiles in an automated way.
|[
:< See _pandora_(1) which is a tool that uses Syd's trace mode to automatically generate sandbox profiles.
|[ *-m config*
:< Configure sandbox during init, may be repeated.
|[ *-p name*
:< Use a sandbox profile during init, may be repeated.
|[ *-P path*
:< Run a configuration file during init, may be repeated.
|[ *-a alias*
:< Set alias of the command. Passed as *argv[0]* to the program.
|[ *-e*
:< Use *-e var=val* to put var=val in the environment for command, may be repeated.
|[
:< Use *-e var* to remove var from the environment for command, may be repeated.
|[
:< Use *-e var=* to pass-through an unsafe environment variable, may be repeated.
|[ *-E mode*
:< Export secure computing rules with the given format to standard output and exit.
|[
:< Mode must be one of *bpf* or *pfc*:
|[
:< *bpf*, aka *Berkeley Packet Filter* is a binary, machine readable format, whereas
|[
:< *pfc*, aka *Pseudo Filter Code* is a textual, human readable format.
|[ *--sh*
:< Output a shell script which defines *esyd* the Syd helper function.
|[
:< Works with POSIX sh, bash and zsh.
|[
:< You may use *eval "$(syd --sh)"* in your shell init file.

# INVOCATION

Syd can either execute a command with the specified arguments or load a
dynamic library and execute the function _syd_main()_ from it. In case
the first non-option argument ends with the suffix ".so", it is expected
to be the path of a dynamic library to load. The dynamic library is
loaded early at the startup in the Syd process such that even its owning
filesystem can be mounted _noexec_, using e.g. bind+/:/:noexec. This
allows to create a very restricted environment for the library function
to run inside. The function _syd_main()_ must take no arguments and
return an integer. This integer is going to be the exit value of the Syd
process. Note, loading a library requires care because its contructors
will run unsandboxed.

# ENVIRONMENT

|[ *SYD_LOG*
:< Set log level to *error*, *warn*, *info*, *debug* or *trace*.
|[ *SYD_LOG_FD*
:< Set log file descriptor. Defaults to *standard error*.
|[ *SYD_NO_SYSLOG*
:< Disable logging to _syslog_(3).
|[
:< By default logs of severity *warn* and higher are logged to _syslog_(3).
|[ *SYD_NO_CROSS_MEMORY_ATTACH*
:< Disable cross memory attach and fallback to */proc/pid/mem*.
|[ *SYD_SH*
:< Pick the shell to spawn when invoked as a login shell, defaults to */bin/sh*.
|[ *SYD_QUICK_BOOT*
:< Enable quick boot mode. This makes Syd startup noticably faster.
|[
:< _However, quick boot removes a layer of defense against some container breaks._
|[
:< Use this if you frequently re-execute _syd_(1) or _syd-oci_(1) as we do on Exherbo during _cave-generate-metadata_(1).

# LOGGING

There're five log levels: error, warn, info, debug, and trace. Log level
may be set with the "SYD_LOG" environment variable. The levels debug and
trace are compiled out if the "log" feature is disabled at compile-time.
Logs go to standard error unless a file descriptor is specified with the
environment variable "SYD_LOG_FD". The messages of severity warn and
above are also sent to _syslog_(3) unless the environment variable
"SYD_NO_SYSLOG" is set.

Syd logs in JSON lines. Below is a list of some of the commonly used
keys and their meanings:

[[ *KEY*
:[ *DESCRIPTION*
|  *abs*
:  True if the socket address in the "unix" field is an abstract UNIX socket
|  *act*
:  Sandbox action (Allow, Warn, Deny, Stop, or Kill)
|  *addr*
:  Network address argument of the syscall (e.g. "127.0.0.1!22")
|  *c*
:  Timestamp as hardware ticks (aarch64 and x86-64 only)
|  *cap*
:  Sandbox capability (r=read, s=stat, w=write, x=exec, i=ioctl, f=force, b=bind, c=connect, m=memory, p=pid, d=safesetid, g=segvguard, t=tpe)
|  *cmd*
:  Process name (or process command line if "log" feature is enabled at compile-time)
|  *ctx*
:  Context of the log entry ("access" for access violations, "scmp" for seccomp events)
|  *cwd*
:  Current working directory of the process
|  *id*
:  Human readable name generated from process ID (since version 3.16.1)
|  *ipv*
:  IP version of the network address in the "addr" field (4 or 6)
|  *l*
:  Log level (1=error, 2=warn, 3=info, 4=debug, 5=trace)
|  *t*
:  Timestamp as seconds since the UNIX Epoch
|  *op*
:  Short description of the current operation
|  *path*
:  Path argument of the syscall
|  *pid*
:  Process ID
|  *src*
:  Origin of the syscall in format "path+offset" (since version 3.23.10. Use, e.g. "objdump -D path | grep offset" to see the corresponding instruction)
|  *sys*
:  Name of the syscall
|  *uid*
:  User ID
|  *unix*
:  UNIX socket address argument of the syscall

# EXIT CODES

Syd exits with the same exit code as the sandbox process itself. If the
sandbox process exits with a signal, Syd exits with 128 plus the value of the
signal. In case there was an error in spawning or waiting for the sandbox
process, Syd exits with *errno* indicating the error condition. E.g. *syd
true* returns *0*, *syd false* returns *1*, and *syd -- syd true* returns *16*
which stands for *EBUSY* which means *Device or resource busy* indicating there
is already a secure computing filter loaded.

# BENCHMARKS

The table below lists the benchmark runs we ran for Syd:

[[ 1: compile kernel
:[ sydbox-{1,3}
:- https://gitlab.exherbo.org/-/snippets/2534
|[ 2: compile kernel
:[ sydbox-{1,3}
:- https://gitlab.exherbo.org/-/snippets/2536
|[ 3: unpack compressed tarball
:[ sydbox-{1,3}, Gentoo sandbox
:- https://gitlab.exherbo.org/-/snippets/2537
|[ 4: compile kernel
:[ sydbox-{1,3}, Gentoo sandbox
:- https://gitlab.exherbo.org/-/snippets/2594
|[ 5: compile kernel in a Podman container
:[ syd-oci, crun, runc, youki, gvisor
:- https://gitlab.exherbo.org/-/snippets/2613
|[ 6: compile kernel in a Podman container
:[ syd-oci, crun, runc, youki, gvisor
:- https://gitlab.exherbo.org/-/snippets/2622

# SEE ALSO

_syd_(2), _syd_(5), _syd_(7), _syd-lock_(1), _syd-ls_(1)

- *syd* homepage: https://sydbox.exherbolinux.org/
- *libsyd* homepage: https://libsyd.exherbolinux.org/
- *pandora* homepage: https://lib.rs/pandora_box
- *paludis* homepage: http://paludis.exherbolinux.org/
- *Landlock* homepage: https://landlock.io
- *Path* wiki: https://en.wikipedia.org/wiki/Path_(computing)
- *Unix domain socket* wiki: https://en.wikipedia.org/wiki/Unix_domain_socket
- *IPv4* wiki: https://en.wikipedia.org/wiki/IPv4
- *IPv6* wiki: https://en.wikipedia.org/wiki/IPv6
- *TOCTOU* wiki: https://en.wikipedia.org/wiki/Time-of-check_to_time-of-use
- *VFS* wiki: https://en.wikipedia.org/wiki/Virtual_file_system
- *ipnetwork* documentation: https://docs.rs/ipnetwork
- *Enabling Logging*: https://docs.rs/env_logger/latest/env_logger/#enabling-logging

# AUTHORS

Maintained by Ali Polatel. Up-to-date sources can be found at
https://gitlab.exherbo.org/sydbox/sydbox.git and bugs/patches can be
submitted to https://gitlab.exherbo.org/groups/sydbox/-/issues. Discuss
in #sydbox on Libera Chat.
