SYD(7)

# NAME

Overview of sandboxing with Syd

# SANDBOXING

There are fourteen sandboxing types:

. *Read sandboxing*
. *Write sandboxing* (and *Path Masking*)
. *Stat sandboxing* (aka *Path Hiding*)
. *Exec sandboxing* (and *SegvGuard*)
. *Ioctl sandboxing* (access *PTY*, *DRM*, *KVM* etc. safely)
. *Force sandboxing* (aka *Verified Execution*, like *Veriexec* & *IntegriForce*)
. *TPE sandboxing* (aka *Trusted Path Execution*)
. *Network sandboxing* (feat. *UNIX*, *IPv4*, *IPv6*, *NetLink* and *KCAPI* sockets)
. *Lock sandboxing* (uses *Landlock LSM*)
. *Proxy sandboxing* (SOCKS proxy forwarding with network namespace isolation, defaults to *TOR*)
. *Memory sandboxing*
. *PID sandboxing* (simpler alternatives to *Control Groups*)
. *SafeSetID* (Safe user/group switching with predefined UID/GID transitions)
. *Ghost mode* (similar to Seccomp Level 1 aka Strict Mode)


Sandboxing may be *on* or *off*:

|[ *off*
:< Sandboxing is off, none of the relevant system calls are checked and all access is granted.
|[ *on*
:< Sandboxing defaults to deny, allowlists and denylists can be used to refine access rights.

In addition, there are filters for every sandboxing to prevent Syd
from reporting an access violation. Note, access is still denied in such
cases.

## Read Sandboxing

This sandboxing checks certain system calls for filesystem read access. If a
system call tries to read a path, this attempt is reported and the system call
is denied. See the section called *Write Sandboxing* for more information on how
to customise this behaviour.

List of filtered system calls are: _open_(2), _openat_(2), _openat2_(2),
_getrandom_(2), and _memfd_create_(2).

- _getrandom_(2) with the flag "GRND_RANDOM" is treated equivalent to \
"open(/dev/random, O_RDONLY)".
- _getrandom_(2) without the flag "GRND_RANDOM" is treated equivalent to \
"open(/dev/urandom, O_RDONLY)".
- _memfd_create_(2) name argument is prepended with "/memfd:" before access \
check. Use e.g. "deny/read+/memfd:\*" to deny access to memory file \
descriptors globally.

## Write Sandboxing

This sandboxing checks certain system calls for filesystem write access. If a system
call tries to write, modify or change attributes of a path, this attempt is reported
in system log and the system call is denied. There are two ways to customise this
behaviour. Syd may be configured to "allow" some _glob_(3p) patterns. If the path
argument of the system call which is subject to be modified matches a pattern in the
list of allowed _glob_(3p) patterns, this attempt is not denied. If, however it
matches a pattern in the list of "deny" _glob_(3p) patterns the attempt is denied
(_last matching pattern wins_). Additionally, Syd may be configured to
"filter" some _glob_(3p) patterns. In this case a match will prevent Syd from
reporting a warning about the access violation, the system call is still denied
though.

List of filtered system calls are: _chmod_(2), _fchmod_(2),
_fchmodat_(2), _fchmodat2_(2), _chown_(2), _chown32_(2), _fchown_(2),
_fchown32_(2), _lchown_(2), _lchown32_(2), _fchownat_(2), _open_(2),
_openat_(2), _openat2_(2), _creat_(2), _mkdir_(2), _mkdirat_(2),
_mknod_(2), _mknodat_(2), _rmdir_(2), _truncate_(2), _truncate64_(2),
_utime_(2), _utimes_(2), _utimensat_(2), _futimesat_(2), _unlink_(2),
_unlinkat_(2), _link_(2), _linkat_(2), _rename_(2), _renameat_(2),
_renameat2_(2), _symlink_(2), _symlinkat_(2), _setxattr_(2),
_fsetxattr_(2), _lsetxattr_(2), _removexattr_(2), _fremovexattr_(2),
_lremovexattr_(2), _ftruncate_(2), _ftruncate64_(2), _fallocate_(2), and
_memfd_create_(2).

- _memfd_create_(2) name argument is prepended with "/memfd:" before access \
check. Use e.g. "deny/write+/memfd:\*" to deny access to memory file \
descriptors globally.

## Stat Sandboxing

This sandboxing checks certain system calls for file metadata access.
This can be one of listing a directory, changing into a directory, or using a
_stat_(2) system call to query file metadata. This sandboxing type may be used to
effectively _hide files and directories_ from the sandbox process.

List of filtered system calls are: _access_(2), _faccessat_(2),
_faccessat2_(2), _getdents_(2), _getdents64_(2), _stat_(2), _fstat_(2),
_lstat_(2), _statx_(2), _newfstatat_(2), _getxattr_(2), _lgetxattr_(2),
_fgetxattr_(2), _listxattr_(2), _flistxattr_(2), _llistxattr_(2),
_statfs_(2), _statfs64_(2), _fstatfs_(2), _fstatfs64_(2),
_fanotify_mark_(2), and _inotify_add_watch_(2).

## Exec Sandboxing

This sandboxing denies _execve_(2), and _execveat_(2) calls in case the
path argument does not match one of the allowed _glob_(3p) patterns.
Note, all _exec_(2) family functions are sandboxed because these
functions are just wrappers of either one of _execve_(2) or
_execveat_(2) system calls. For scripts, access check is done for both
the script and the hashbang program.

As of version 3.16.3, Syd checks the paths of the dynamic libraries
an executable is linked against for exec access as well. This only
works for ELF files.

As of version 3.21.1, Syd checks the _memfd_create_(2) system call for
exec access as well because these files are created with mode 777.
_memfd_create_(2) name argument is prepended with "/memfd:" before
access check. Use e.g. "deny/exec+/memfd:\*" to deny access to memory
file descriptors globally.

As of verison 3.21.2, Syd seals memory file descriptors as
non-executable by default. Therefore memory file descriptors are not
checked for Exec access unless the option `trace/allow_unsafe_memfd:1`
is set.

As of version 3.21.3, Syd hooks into _mmap_(2), and _mmap2_(2) system
calls and checks the file descriptor for Exec access when the memory
protection mode includes "PROT_EXEC" and flags does not include
"MAP_ANONYMOUS" which typically indicates a _dlopen_(3). Therefore
libraries dynamically loaded at runtime are checked for Exec access as
well.

## Ioctl Sandboxing

This sandboxing checks the _ioctl_(2) system call for filesystem access.
In addition, _ioctl_(2) requests can be allowlisted/denylisted by adding
them to the respective list using "ioctl/allow+" and "ioctl/deny+".

## SegvGuard

As of version syd-3.16.3, Syd has a simple implementation of SegvGuard.
The implementation is inspired by that of HardenedBSD with identical
defaults: If a sandbox process produces a core dump for
"segvguard/maxcrashes" times (defaults to 5), in a period of
"segvguard/expiry" seconds (defaults to 2 minutes), subsequent attempts
to execute the same executable is denied for "segvguard/suspension"
seconds (defaults to 10 minutes). SegvGuard can be disabled either by
setting "segvguard/expiry:0" or by setting "trace/allow_unsafe_exec:1".
The former setting works at runtime whereas the latter must be passed
on startup. SegvGuard is guarded by the exec-TOCTOU mitigator.

Check out the following links for further information on SegvGuard:

- http://en.wikibooks.org/wiki/Grsecurity/Appendix/Grsecurity_and_PaX_Configuration_Options#Deter_exploit_bruteforcing
- http://en.wikibooks.org/wiki/Grsecurity/Appendix/Grsecurity_and_PaX_Configuration_Options#Active_kernel_exploit_response
- http://phrack.org/archives/issues/59/9.txt
- http://phrack.org/archives/issues/58/4.txt
- https://github.com/HardenedBSD/hardenedBSD/wiki/segvguard2-ideas---brainstorm

## Force Sandboxing

Force Sandboxing enhances system security by scrutinizing the path
provided to _execve_(2) and _execveat_(2) system calls, comparing them
against a predefined Integrity Force map -- a registry of
path-to-checksum correlations. Upon invocation of these calls, the
sandbox computes the checksum of the target binary and cross-references
it with the map. Discrepancies trigger rule-defined actions: execution
might proceed with a logged warning, or culminate in the termination of
the process in violation. This mechanism allows for rigorous enforcement
of binary integrity, echoing the preventative ethos of HardenedBSD's
Integriforce and NetBSD's Veriexec by proactively mitigating
unauthorised code execution, albeit with a unique emphasis on flexible,
user-defined consequence management ranging from permissive alerts to
stringent execution blocks.

Distinguishing itself through user-centric customization, Force Sandboxing
offers a versatile approach to execution integrity. Administrators can tailor
the sandbox's response to checksum mismatches -- kill, or warn -- thereby
balancing security needs with operational flexibility. This adaptability,
combined with tools like _syd-sha_(1) for checksum calculation and _syd-path_(1)
for rule creation, positions Force Sandboxing as a powerful ally in the
preservation of system integrity. See _force_ command in _syd_(2) manual
page on how to add/remove entries to/from the Integrity Force map.

As of version 3.16.3, Syd checks the paths of the dynamic libraries an
executable is linked against for force access as well. This only works
for ELF files.

As of version 3.21.3, Syd hooks into _mmap_(2), and _mmap2_(2) system
calls and checks the file descriptor for Force access when the memory
protection mode includes "PROT_EXEC" and flags does not include
"MAP_ANONYMOUS" which typically indicates a _dlopen_(3). Therefore
libraries dynamically loaded at runtime are checked for Force access as
well.

## TPE sandboxing

As of version 3.21.0, Syd introduces Trusted Path Execution (TPE)
sandboxing, which restricts the execution of binaries to ensure they
come from trusted directories. The intention is to make privilege
escalation harder when an account restricted by TPE is compromised as
the attacker won't be able to execute custom binaries which are not in
the trusted path. Trusted directories are defined as those that meet the
following criteria:

. Not writable by group or others.
. Optionally owned by root (controlled by the "tpe/root_owned" setting).
. Optionally owned by the current user or root (controlled by the "tpe/user_owned" setting).

If the directories do not meet these criteria, the execution is denied
with an "EACCES" error, and optionally, the offending process can be
terminated with "SIGKILL" using the "default/tpe:kill" setting. This
mechanism ensures that only binaries from secure, trusted paths can be
executed, enhancing security by preventing unauthorized code execution.
TPE sandboxing operates by checking the parent directories of
executables at three stages:

. During the system call entry of _execve_(2) and _execveat_(2) to check scripts.
. On _ptrace_(2) exec event to check the ELF executable.
. On _mmap_(2) when a dynamic library is loaded into the memory.

TPE can be configured to apply to a specific user group. By default, TPE
applies to all users. However, administrators can specify an untrusted
GID with the "tpe/gid" setting, restricting TPE only to users in that
group. Additionally, TPE can negate GID logic with the "tpe/negate"
setting, making the specified group trusted and exempt from TPE.

Syd's TPE implementation is based on HardenedBSD's which is inspired
by GrSecurity's TPE. Check out the following links for more information:

- http://phrack.org/issues/52/6.html#article
- http://phrack.org/issues/53/8.html#article
- https://wiki.gentoo.org/wiki/Hardened/Grsecurity_Trusted_Path_Execution

## Network Sandboxing

This sandboxing exposes a way to prevent unwanted network calls. The filtered
system calls are: _bind_(2), _connect_(2), _sendto_(2), _sendmsg_(2), and
_sendmmsg_(2). To increase usability, these system calls are filtered in two
groups: "bind" and "connect". _bind_(2) belongs to the "bind" group, whereas the
other system calls belong to the "connect" group. See the section called *Write
Sandboxing* for more information on how to customise this behaviour.

## Lock Sandboxing

This sandboxing utilises the *Landlock Linux Security Module* for simple
unprivileged access control. This sandboxing type is not dynamic and is applied
at the kernel level on startup. The sandboxing may be turned on with the
"sandbox/lock:on" sandbox command, and _read-only_, and _read-write_ allowlists
can be populated using the sandbox commands "allow/lock/read+/path" and
"allow/lock/write+/path". Note the Syd process is also included in this
sandbox for added security such that a compromised Syd process is still stuck
inside the *Landlock* sandbox.

## Proxy Sandboxing

As of version 3.22.0, Proxy sandboxing in Syd confines network communication
exclusively through a designated SOCKS proxy, enforced by the helper utility
_syd-tor_(1). Configured at startup with "sandbox/proxy:on", this type implies
the use of "unshare/net:1", isolating network namespaces to prevent direct
network access. Traffic is forwarded from a specified local port
(proxy/port:9050) to an external address and port (proxy/ext/addr:127.0.0.1,
proxy/ext/port:9050). This setup ensures all network interactions route through
the proxy, leveraging zero-copy data transfers and edge-triggered _epoll_(7) for
efficient event handling. The implementation enhances security by employing
seccomp and Landlock for additional confinement, preventing unauthorized network
access and ensuring strict adherence to the defined network path. This approach
minimizes the risk of proxy bypasses and maintains the integrity of the network
isolation.

## Memory Sandboxing

This sandboxing handles the system calls _brk_(2), _mmap_(2), _mmap2_(2),
and _mremap_(2) and checks the per-process memory usage on each memory
allocation request. If the memory usage reaches the maximum value
defined by *mem/max*, the system call is denied with *ENOMEM*. Moreover
the virtual memory size can be limited using *mem/vm_max*. If the limit
is reached on the entry of any of the respective system calls, the
system call is denied with *ENOMEM*. If *mem/kill* is set to
true, the signal *SIGKILL* is delivered to the offending process.

The per-process memory usage is a fair estimate calculated using the file
*/proc/pid/smaps* summing the following fields together:

. *"Pss" (Proportional Set Size)* is similar to *Rss*, but \
accounts for shared memory more accurately by dividing it \
among the processes that share it.
. *"Private_Dirty"* represents the private memory that has \
been modified (dirty).
. *"Shared_Dirty"* represents the shared memory that has \
been modified.

Note, *"Rss" (Resident Set Size)* is the portion of memory occupied by a
process that is held in RAM.

Note, _memory sandboxing is not an alternative to cgroups_. You should use
_cgroups_(7) when you can instead. This is meant for more constrained
environments where _cgroups_(7) is not supported or not available due to missing
permissions or other similar restrictions.

Warning, _this is not a strict limit_, Syd is reliant on the allocation
granularity of the processes in the sandbox. Programs will be able to overstep
the limit at least by amount of memory they allocate at once.

## PID sandboxing

This sandboxing handles the system calls _fork_(2), _vfork_(2), _clone_(2), and
_clone3_(2) and checks the total number of tasks running on the system on each
process creation request. If the count reaches the maximum value defined by
*pid/max*, the system call is denied with "EAGAIN". If "pid/kill" is set to
true, the signal "SIGKILL" is delivered to the offending process.

This sandboxing type is best coupled with a pid namespace using "unshare/pid".
In this mode, Syd will check the number of running tasks _in the current
namespace only._

Note, _PID sandboxing is not an alternative to cgroups_. You should use
_cgroups_(7) when you can instead. This is meant for more constrained
environments where _cgroups_(7) is not supported or not available due to missing
permissions or other similar restrictions.

## SafeSetID

*SafeSetID*, introduced in version 3.16.8, enhancing the management of
UID/GID transitions by replacing the previous "trace/allow_unsafe_setid"
configuration. This feature enables finer-grained control by allowing
administrators to explicitly specify permissible transitions for UID and
GID changes, thus tightening security constraints around process
privilege management. *SafeSetID* is enabled by default, but typically
requires the user to keep the "CAP_SETUID" or the "CAP_SETGID"
capability in addition with the options "trace/allow_safe_setuid:1", or
"trace/allow_safe_setgid:1" on Syd startup. It works by allowing
predefined UID and GID transitions that are explicitly configured using
the "setuid+<source_uid>:<target_uid>" and
"setgid+<source_gid>:<target_gid>" commands in the Syd configuration.
This ensures that transitions can only occur between specified user and
group IDs, and unauthorised privilege escalations are blocked. For
instance, a transition might be allowed from a higher-privileged user to
a less-privileged user but not vice versa, thereby preventing any
escalation of privileges through these system calls.

Since version 3.24.5, Syd applies a kernel-level _seccomp_(2) filter by
default to deny all set\*uid system calls with UID less than or equal to
11 (aka *operator*), and all set\*gid system calls with GID less than or
equal to 14 (aka *uucp*). This means even a compromised Syd process
cannot elevate privileges using these system calls. See "syd-ls setid"
to see the full list of system calls in this group.

Note, with the options "trace/allow_safe_setuid:1", and
"trace/allow_safe_setgid:1", Syd will keep the "CAP_SETUID" and
"CAP_SETGID" capabilities and sandbox process will inherit these
capabilities from Syd. Since version 3.24.6, Syd drops the "CAP_SETUID"
capability after the first successful UID transition and similarly the
"CAP_SETGID" capability after the first successful GID transition. This
means Syd can only ever change its UID and GID once in its lifetime.
However, this does not completely lock the setid system calls in the
sandbox process: _Transitions to Syd's current UID and GID are continued
in the sandbox process_ which means the first successful UID and GID
transition will continue to function as long as the sandbox process
keeps the respective "CAP_SETUID", and "CAP_SETGID" capabilities. This
allows containing daemons, such as nginx, which spawn multiple
unprivileged worker processes out of a single main privileged process.

## Ghost mode

Ghost Mode, introduced in Syd version 3.20.0, is the final sandboxing
type, closely resembling Seccomp Level 1, also known as Strict Mode.
This mode enhances security by allowing a process to transition to a
highly restrictive state after completing its initial setup. When a
sandboxed process is ready for this higher level of confinement, it
invokes Ghost Mode by executing the "ghost" command using the virtual
stat API, e.g. with stat("/dev/syd/ghost", NULL). This system call
invocation is going to fail with the errno "EOWNERDEAD" if Ghost mode is
successfully initiated. Upon receiving this command, Syd closes the
_seccomp_(2) notify file descriptor. This action elevates all previously
hooked system calls to a kernel-level deny with the "ENOSYS" errno,
effectively making them unavailable. The transition to Ghost Mode is
irreversible; once the file descriptor is closed, the process is locked
into this restricted state. This mechanism ensures that the sandboxed
process can only perform a very limited set of operations, akin to those
allowed in Seccomp Level 1, thus significantly reducing its potential
attack surface. Ghost Mode provides a robust security measure by denying
all but the most essential system calls, which is crucial for
applications that require maximum isolation and security after their
initial configuration phase.

The mode is aptly named "Ghost" because, upon closing the _seccomp_(2)
notify file descriptor, the sandboxed process effectively detaches from
Syd and becomes independent, much like a ghost. From this point onwards,
Syd no longer intervenes or manages the process's system calls but
merely waits for the process to exit. This detachment underscores the
finality and isolation of the Ghost Mode, ensuring that the process
operates in a secure, tightly confined environment without further
interaction from Syd. This mechanism is particularly useful for
processes that require maximum security and minimal system call exposure
after their initial configuration phase, providing a robust layer of
protection against various exploits and vulnerabilities.

# SECURITY

Syd stands out for its ability to operate without requiring elevated
privileges, eliminating the need for root access. This feature
significantly simplifies setup and usage. Users benefit from the
capability to dynamically configure the sandbox from within, with
options to secure it further as needed. *Tip*: To take a quick peek at
the seccomp filters applied by Syd under various different
configurations, use *syd <flags...> -Epfc*.

## Threat Model

Syd strictly adheres to the current threat model of Seccomp. The goal is
to restrict how untrusted userspace applications interact with the
shared OS kernel through system calls to protect the kernel from
userspace exploits (e.g., shellcode or ROP payload). The kernel is
trusted. Syd's threat model delineates the sandbox as the trusted
interceptor of system calls, while all user applications running within
the sandbox are considered untrusted. These untrusted applications can
manipulate their execution environment through syscalls, and attackers
are assumed to have the capability to execute arbitrary code within
these applications.  Syd uses several mechanisms, including seccomp and
ptrace for syscall filtering, Landlock for filesystem access
restrictions, and namespaces for process and device isolation, to limit
the impact of these potential attacks. The threat model assumes that
attackers have control over the untrusted user space and may attempt
reads, writes, or arbitrary code execution that could influence the
behavior of the trusted sandbox or exploit syscall handling. The
security of Syd relies on the correctness of its implementation and the
underlying Linux kernel features it utilizes. It is assumed that there
are no vulnerabilities in Syd's interception and handling of syscalls,
nor in the enforcement mechanisms provided by Landlock and namespaces.
External attacks via network vectors or physical access to hardware are
considered out of scope for this threat model.

## Accessing remote process memory

Syd denies various system calls which can access remote process memory such
as _ptrace_(2) and _process_vm_writev_(2) and common sandboxing profiles such as
"paludis" and "user" disallow write access to the "/proc/pid/mem" file. This
makes "TOCTOU" attack vectors harder to realise. See "syd-ls deny" to see
the full list of denied system calls.

## Enhanced Handling of PTRACE_TRACEME

As of version 3.16.3, Syd introduced a new feature for managing the
"PTRACE_TRACEME" operation, aimed at improving stealth against detection.
Traditionally, "PTRACE_TRACEME" is the only _ptrace_(2) operation allowed by a
tracee, which makes it a common target for detection of ptracers. By converting
"PTRACE_TRACEME" into a no-operation (no-op) that always succeeds, Syd aims to
subtly prevent simple detection methods that rely on this operation.
Additionally, other _ptrace_(2) operations are modified to return an EPERM
(Permission Denied) error instead of ENOSYS (Function Not Implemented), which
helps reduce the likelihood of the sandbox being detected through these errors.
This approach enhances the discreetness of Syd's operation by mitigating
straightforward detection tactics used by monitored processes.

As of version 3.19.0, Syd extends this mitigation and turns the system
call _ptrace_(2) into a no-op. Again, this provides a best-effort
mitigation against using requests such as "PTRACE_ATTACH" or
"PTRACE_SEIZE" to detect a ptracer.

Note, this mitigation is simple and zero-cost, however a clever _ptrace_(2)
detector can bypass it with e.g. a double _ptrace_(2) as exemplified here:
https://arxiv.org/pdf/2109.06127

```
if(ptrace(PTRACE_TRACEME)==0 && ptrace(PTRACE_TRACEME)==-1){
	evade();
}
```

Since this example relies on internal function states and side-effects,
it bypasses Syd's mitigation. In such cases, user may opt for the option
"trace/allow_unsafe_exec:1", when Syd will not use _ptrace_(2) _at all_,
hence there is going to be no ptracer to detect for the malware with the
logic bomb.

## Hardened procfs

As of version 3.14.5, Syd has introduced a significant enhancement to its
sandbox tool, adopting a process hardening approach similar to that utilised by
HardenedBSD's handling of procfs/linprocfs. This new security measure
specifically targets and restricts access to a set of critical paths within the
process file system, including the files "auxv", "environ", "maps", "mem",
"pagemap", "personality", "stack", "syscall", and "wchan" under the directories
/proc/$pid and /proc/$pid/task/$tid. Access to these paths is denied by
default, regardless of the process or thread identifier (PID/TID) involved or
the state of sandboxing. This strict policy of unconditional denial is designed
to further secure the system by preventing unauthorised access to sensitive
process information, thereby enhancing the overall integrity and security
posture of the environment. This addition complements existing protections
against remote process memory access. Together, these measures significantly
mitigate the risk of _Time-of-Check to Time-of-Use (TOCTOU)_ attack vectors,
underscoring Syd's commitment to providing a robust security framework for its
users. As of version 3.15.1, Syd further hardens the procfs restrictions by
denying access to the files /proc/cmdline, /proc/config.gz, /proc/dma,
/proc/interrupts, /proc/iomem, /proc/ioports, /proc/kallsyms,
/proc/kcore, /proc/key-users, /proc/keys, /proc/modules, /proc/mtrr,
/proc/partitions, /proc/softirqs, /proc/sysrq-trigger, /proc/version,
/proc/vmallocinfo and directories /proc/fs, /proc/irq, and /proc/tty
regardless of the sandboxing mode.

## Restricted Device Access

As of version 3.15.1, Syd introduces a critical security feature that
significantly bolsters the integrity and confidentiality of the running kernel
and system devices. Building upon the foundational principles of the hardened
procfs feature, this update extends protection mechanisms to critical device
files, specifically targeting "/dev/mem", "/dev/kmem", "/dev/port", and
"/dev/cpu/\*". This enhancement prevents both reading from and writing to these
sensitive areas, effectively safeguarding the system against unauthorised
modifications or leaks of kernel memory and preventing misuse of system ports
and CPU-specific registers.

## Denying TIOCLINUX ioctl

The limitation on the use of the "TIOCLINUX" _ioctl_(2) within secure
environments, similar to the Syd sandbox, is an essential security measure
addressing vulnerabilities specific to Linux terminal operations. The
"TIOCLINUX" _ioctl_(2) command offers various functionalities, including but not
limited to manipulating console settings, changing keyboard modes, and
controlling screen output. While these capabilities can be leveraged for
legitimate system management tasks, they also introduce potential security
risks, particularly in multi-user environments or in the context of sandboxed
applications.

The security concerns surrounding "TIOCLINUX" stem from its ability to alter
terminal behaviors and settings in ways that could be exploited for unauthorised
information disclosure, terminal hijacking, or privilege escalation. For
instance, manipulating the console display could mislead users about the true
nature of the operations being executed, or altering keyboard settings could
capture or inject keystrokes.

In summary, the restriction on "TIOCLINUX" within secure environments is a vital
security strategy, addressing the complex risks associated with direct terminal
manipulation capabilities. This precaution is in keeping with the broader
security community's efforts to mitigate known vulnerabilities and enhance the
security posture of systems handling sensitive processes and data.

## Denying TIOCSTI ioctl

The restriction on the use of the "TIOCSTI" _ioctl_(2) within the Syd
sandbox addresses a significant security vulnerability associated with
terminal input injection. The "TIOCSTI" _ioctl_(2) allows a byte to be
inserted into the terminal input queue, effectively simulating keyboard
input. This capability, while potentially useful for legitimate
purposes, poses _a substantial security risk_, especially in scenarios
where a process might retain access to a terminal beyond its intended
lifespan. Malicious use of this _ioctl_(2) can lead to the injection of
commands that execute with the privileges of the terminal's owning
process, thereby breaching the security boundaries intended by user
permissions and process isolation mechanisms. The concern over "TIOCSTI"
is well-documented in the security community. For example, OpenBSD has
taken measures to mitigate the risk by disabling the "TIOCSTI"
_ioctl_(2), reflecting its stance on the _ioctl_(2) as _one of the most
dangerous_ due to its potential for abuse in command injection attacks.
The decision to disable or restrict "TIOCSTI" in various Unix-like
operating systems underscores the _ioctl_(2)'s inherent security
implications, particularly in the context of privilege escalation and
the execution of unauthorised commands within a secured environment.

In summary, the restriction on "TIOCSTI" within Syd is a critical
security measure that prevents a class of vulnerabilities centered
around terminal input injection, safeguarding against unauthorised
command execution and privilege escalation. This precaution aligns with
broader security best practices and mitigations adopted by the security
community to address known risks associated with terminal handling and
process isolation.

## Denying FS_IOC_SETFLAGS ioctl

As of version 3.24.2, Syd denies the "FS_IOC_SETFLAGS" _ioctl_(2)
request by default, a critical security measure to ensure that once file
flags are set, they remain unchanged throughout the runtime of the
sandbox. This policy is particularly focused on the *immutable* and
*append-only* flags, which need to be configured by an administrator at
the start of the Syd process. Once these attributes are set on crucial
system and log files -- marking them either as immutable to prevent any
modification, or append-only to ensure that existing data cannot be
erased -- they are frozen. This means that no subsequent modifications
can be made to these attributes, effectively locking down the security
settings of the files against any changes. This approach prevents
scenarios where, even after a potential security breach, malicious
entities are unable to alter or delete important files, thus maintaining
the integrity and reliability of the system against tampering and
ensuring that audit trails are preserved.

## Denying PR_SET_MM prctl

The "PR_SET_MM" _prctl_(2) call allows processes with the "CAP_SYS_RESOURCE"
capability to adjust their memory map descriptors, facilitating operations like
self-modifying code by enabling dynamic changes to the process's memory layout.
For enhanced security, especially in constrained environments like Syd, this
capability is restricted to prevent unauthorised memory manipulations that could
lead to vulnerabilities such as code injection or unauthorised code execution.
Notably, Syd proactively drops "CAP_SYS_RESOURCE" among other capabilities at
startup to minimise security risks. This action is part of Syd's broader
security strategy to limit potential attack vectors by restricting process
capabilities. Users can query "syd-ls drop" for a comprehensive list of
capabilities that Syd relinquishes upon initialization, reinforcing the
sandbox's commitment to a secure execution environment by adhering to the
principle of least privilege.

## Restricting prctl option space and trace/allow_unsafe_prctl

Syd meticulously confines the scope of permissible _prctl_(2) operations to
enhance security within its sandbox environment. By limiting available
_prctl_(2) options to a specific set, including but not limited to
"PR_SET_PDEATHSIG", "PR_GET_DUMPABLE", "PR_SET_NO_NEW_PRIVS", and
"PR_SET_SECCOMP", Syd ensures that only necessary process control
functionalities are accessible, thereby reducing the risk of exploitation
through less scrutinised _prctl_(2) calls. This constraint is pivotal in
preventing potential security vulnerabilities associated with broader _prctl_(2)
access, such as unauthorised privilege escalations or manipulations of process
execution states. However, recognizing the need for flexibility in certain
scenarios, Syd offers the option to lift these restrictions through the
"trace/allow_unsafe_prctl:1" setting. This capability allows for a tailored
security posture, where users can opt for a more permissive _prctl_(2)
environment if required by their specific use case, while still maintaining
awareness of the increased security risks involved.

## Restricting io_uring interface and trace/allow_unsafe_uring

The _io_uring_(7) interface can be used to _bypass path sandboxing_. By default,
Syd restricts _io_uring_(7) operations due to their ability to perform system
calls that could undermine the sandbox's security controls, particularly those
designed to limit file access and modify file permissions. The setting,
"trace/allow_unsafe_uring", when enabled, relaxes these restrictions, allowing
_io_uring_(7) operations to proceed unimpeded. While this can significantly
enhance I/O performance for applications that rely on _io_uring_(7) for
efficient asynchronous operations, it requires careful consideration of the
security implications, ensuring that its use does not inadvertently compromise
the sandboxed application's security posture. Use "syd-ls uring" to see the full
list of system calls that belong to the io_uring interface.

## Restricting creation of device special files

Since version 3.1.12, Syd has enhanced its security model by disallowing
the creation of device special files through the _mknod_(2) and
_mknodat_(2) system calls. This decision is rooted in mitigating
potential security vulnerabilities, as device special files could be
exploited to circumvent established path-based access controls within
the sandbox environment. These files, which include character and block
devices, can provide direct access to hardware components or facilitate
interactions with kernel modules that could lead to unauthorised actions
or data exposure. By restricting their creation, Syd significantly
reduces the risk of such exploit paths, reinforcing the integrity and
security of the sandboxed applications. This measure ensures that only
predefined types of files -- such as FIFOs, regular files, and sockets --
are permissible, aligning with the principle of least privilege by
limiting file system operations to those deemed safe within the
sandbox's context.

## Sharing Pid namespace with signal protections

Since version 3.6.7, Syd has introduced a crucial security feature that
prevents sandboxed processes from sending signals to the Syd process or
any of its threads. This protection is implemented by hooking and
monitoring system calls related to signal operations, including
_kill_(2), _tkill_(2), _tgkill_(2), and _pidfd_open_(2). When a
sandboxed process attempts to send a signal to Syd or its threads, these
system calls are intercepted, and the operation is denied at the seccomp
level with an "EACCES" error. This measure ensures that Syd maintains
control over the execution and management of sandboxed processes,
safeguarding against interruptions or unauthorised interactions that
could compromise the security or stability of the sandbox environment.
This security mechanism is part of Syd's broader strategy to share the
same root, private proc, and mount namespaces with the sandboxed
process, facilitating secure and simple system call emulation. By making
Syd and its threads immune to signals from sandboxed processes, the
integrity and isolation of the sandboxed environment are significantly
enhanced, preventing potential exploitation scenarios where sandboxed
processes could disrupt the operation of the sandbox manager or
interfere with other sandboxed processes.

## Simplified Filesystem Access for Legacy Daemons

Since version 3.12.0, Syd has adopted a nuanced approach to managing
filesystem access controls, specifically concerning the _chroot_(2) and
_pivot_root_(2) system calls. In Syd, these calls are treated as
no-operations (no-ops), meaning they _report success without performing
any action_. This decision is strategically aimed at accommodating older
daemons that invoke "chroot(/var/empty)" as part of their initialization
sequence, typically after completing necessary path accesses. By
rendering _chroot_(2) and _pivot_root_(2) as no-ops at the seccomp-bpf
kernel level, Syd ensures compatibility with such legacy software
without introducing complexities in syscall emulation.

The rationale behind this approach stems from the necessity of
maintaining a unified filesystem root between the sandbox and the
sandboxed processes to simplify syscall emulation. However, certain
daemons, like OpenNTPD, traditionally employ _chroot_(2) to a minimal
directory (e.g., "/var/empty") as a security measure. Syd's no-op policy
for these system calls allows it to host these daemons without
necessitating alterations to their source code or operational
logic, thus broadening the utility of Syd in securing legacy
applications without sacrificing operational integrity or
complicating the sandbox's implementation.

## Streamlining File Synchronization Calls

Since version 3.8.8, Syd has rendered the _sync_(2) and _syncfs_(2)
system calls as no-operations (no-ops), ensuring they report success
without executing any underlying functionality. This adjustment is
designed to streamline operations within the sandboxed environment,
bypassing the need for these file synchronization actions that could
otherwise impact performance or complicate the sandbox's control over
file system interactions. By adopting this approach, Syd enhances its
compatibility with applications that issue these calls, without altering
the sandboxed process's behavior or the integrity of file system
management.

## Restricting Resource Limits, Core Dumps, and trace/allow_unsafe_prlimit

Since version 3.9.6, Syd has implemented restrictions on setting process
resource limits and generating core dumps for the sandboxed process,
enhancing the sandbox's security posture. This measure prevents the
sandboxed process from altering its own resource consumption boundaries
or producing core dumps, which could potentially leak sensitive
information or be exploited for bypassing sandbox restrictions. However,
recognizing the need for flexibility in certain use cases, Syd provides
the option to disable these restrictions at startup through the
"trace/allow_unsafe_prlimit:1" setting. This allows administrators to
tailor the sandbox's behavior to specific requirements, balancing
security considerations with functional needs.

## Enhancing Sandbox Security with Landlock

Since version 3.0.1, Syd leverages the *Landlock Linux Security Module
(LSM)* to enforce advanced filesystem sandboxing, significantly
bolstering the security framework within which sandboxed processes
operate. By integrating Landlock, Syd empowers even unprivileged
processes to create secure sandboxes, enabling fine-grained access
control over filesystem operations without requiring elevated
permissions. This approach is instrumental in mitigating the risk of
security breaches stemming from bugs or malicious behaviors in
applications, offering a robust layer of protection by restricting
ambient rights, such as global filesystem or network access. Landlock
operates by allowing processes to self-impose restrictions on their
access to system resources, effectively creating a secure environment
that limits their operation to a specified set of files and directories.
This mechanism is particularly useful for running legacy daemons or
applications that require specific environmental setups, as it allows
for the precise tailoring of access rights, ensuring processes can only
interact with designated parts of the filesystem. For instance,
by setting Landlock rules, Syd can confine a process's
filesystem interactions to read-only or read-write operations on
explicitly allowed paths, thus preventing unauthorised access to
sensitive areas of the system.

Furthermore, the inclusion of the Syd process itself within the
Landlock-enforced sandbox adds an additional layer of security. This
design choice ensures that even if the Syd process were compromised, the
attacker's ability to manipulate the sandboxed environment or access
unauthorised resources would be significantly constrained. This
self-sandboxing feature underscores Syd's commitment to maintaining a
high security standard, offering peace of mind to users by ensuring
comprehensive containment of sandboxed processes.

## Namespace Isolation in Syd

Syd enhances sandbox isolation through meticulous namespace use, starting
from version 3.0.2. Version 3.9.10 marks a pivotal enhancement by restricting
user subnamespace creation, addressing a key path sandboxing bypass
vulnerability. This strategic limitation thwarts sandboxed processes from
altering their namespace environment to access restricted filesystem areas.
Furthermore, since version 3.11.2, Syd maintains process capabilities within
user namespaces, mirroring the _unshare_(1) command's "--keep-caps" behavior.
This ensures sandboxed processes retain necessary operational capabilities,
enhancing security without compromising functionality. Additionally, Syd
utilises the powerful "bind" command within the mount namespace to create
secure, isolated environments by allowing specific filesystem locations to be
remounted with custom attributes, such as "read-only" or "no-exec", providing a
flexible tool for further restricting sandboxed processes' access to the
filesystem.

Syd also introduces enhanced isolation within the mount namespace by
offering options to bind mount temporary directories over "/dev/shm" and
"/tmp", ensuring that sandboxed processes have private instances of these
directories. This prevents inter-process communication through shared memory
and mitigates the risk of temporary file-based attacks, further solidifying the
sandbox's defence mechanisms. Moreover, since version 3.11.2, Syd mounts
the proc filesystem privately with the "hidepid=2" option, enhancing privacy by
concealing process information from unauthorised users.

Syd's "container" and "immutable" profiles exemplify its adaptability,
offering from isolated to highly restrictive environments. The "container"
profile provides a general-purpose sandbox, while the "immutable" profile
enforces stricter controls, such as making essential system directories
read-only, to prevent tampering. This comprehensive approach underlines Syd's
adept use of kernel features for robust sandbox security, ensuring a secure and
controlled execution environment for sandboxed applications. See "syd-cat
container", and "syd-cat immutable" to list the rules in these sandboxing
profiles.

As of version 3.23.0, Syd has further strengthened its security. The
introduction of a time namespace, represented by the "unshare/time:1"
option, allows Syd to reset the boot-time clock, ensuring that the
_uptime_(1) command reports container uptime instead of host uptime.
Moreover, the creation of namespaces, including mount, UTS, IPC, user,
PID, net, cgroup, and time is denied by default to prevent unauthorized
namespace manipulation that could undermine path sandboxing security. To
allow specific namespace types, administrators must explicitly enable
them via the "trace/allow_unsafe_namespace" setting. Another restriction
to note is that the system calls _mount_(2), _mount_setattr_(2),
_umount_(2), and _umount2_(2) are denied by default unless "mount"
namespace is allowed. This change ensures tighter control over process
capabilities and isolation, reinforcing the defense mechanisms against
potential security breaches.

## Restricting environment and trace/allow_unsafe_env

Syd, from version 3.11.1, has implemented measures to clear unsafe
environment variables, such as "LD_PRELOAD", enhancing security by preventing
the manipulation of dynamic linker behavior by sandboxed processes. This action
mitigates risks associated with dynamic linker hijacking, where adversaries may
load malicious shared libraries to execute unauthorised code, potentially
leading to privilege escalation, persistence, or defence evasion. Variables like
"LD_PRELOAD" allow specifying additional shared objects to be loaded before any
others, which could be exploited to override legitimate functions with malicious
ones, thus hijacking the execution flow of a program. To accommodate scenarios
where developers might need to use these variables for legitimate purposes,
Syd allows this security feature to be disabled at startup with
"trace/allow_unsafe_env:1", offering flexibility while maintaining a
strong security posture. This careful balance ensures that sandboxed
applications operate within a tightly controlled environment, significantly
reducing the attack surface and enhancing the overall security framework within
which these applications run. See "syd-ls env" to see the full list of
environment variables that Syd clears from the environment of the sandbox
process.

## Managing Linux Capabilities for Enhanced Security

Since its 3.0.17 release, Syd strategically curtails specific Linux
_capabilities_(7) for sandboxed processes to bolster security. By revoking privileges
such as "CAP_SYS_ADMIN" among others, Syd significantly reduces the risk of
privilege escalation and system compromise. This proactive measure ensures that
even if a sandboxed process is compromised, its ability to perform sensitive
operations is severely limited. The comprehensive list of dropped capabilities,
including but not limited to "CAP_NET_ADMIN", "CAP_SYS_MODULE", and
"CAP_SYS_RAWIO", reflects a meticulous approach to minimizing the attack surface.
See "syd-ls drop" to see the full list of _capabilities_(7) that Syd drops at
startup.

Exceptions to this stringent policy, introduced in version 3.11.1, such
as retaining "CAP_NET_BIND_SERVICE" with "trace/allow_unsafe_bind:1",
"CAP_NET_RAW" with "trace/allow_unsafe_socket:1", "CAP_SYSLOG" with
"trace/allow_unsafe_syslog:1" and "CAP_SYS_TIME" with
"trace/allow_unsafe_time:1", offer a nuanced security model. These
exceptions allow for necessary network, syslog and time adjustments
within the sandbox, providing flexibility without significantly
compromising security.

Since version 3.12.5, Syd allows the user to prevent dropping capabilities at
startup using the command "trace/allow_unsafe_caps:1". This command may be used to
construct privileged containers with Syd.

This balanced strategy of restricting _capabilities_(7), coupled with selective
permissions, exemplifies Syd's commitment to crafting a secure yet functional
sandbox environment. By leveraging the granularity of Linux _capabilities_(7),
Syd offers a robust framework for safeguarding applications against a variety
of threats, underscoring its role as a pivotal tool in the security arsenal of
Linux environments.

## Enhanced Symbolic Link Validation

As of version 3.13.0, Syd enhances security by enforcing stricter
validation on symbolic links within "/proc/pid/fd", "/proc/pid/cwd",
"/proc/pid/exe", and "/proc/pid/root", addressing potential misuse in
container escape scenarios. Specifically, Syd returns an "ELOOP" error
for attempts to resolve these symbolic links if they do not pertain to
the _current process_, akin to implementing "RESOLVE_NO_MAGICLINKS"
behavior of the _openat2_(2) system call. This measure effectively
hardens the sandbox against attacks exploiting these links to access
resources outside the intended confinement, bolstering the isolation
provided by Syd and mitigating common vectors for privilege escalation
and sandbox escape. As of version 3.14.5, Syd keeps intercepting
path system calls even if sandboxing is off making this protection
unconditional.

## Memory-Deny-Write-Execute Protections

Syd version 3.14.1 enhances its security framework by implementing
Memory-Deny-Write-Execute (MDWE) protections, aligning with the "PR_SET_MDWE"
and "PR_MDWE_REFUSE_EXEC_GAIN" functionality introduced in Linux kernel 6.3.
This feature establishes a stringent policy against creating memory mappings
that are _simultaneously writable and executable_, closely adhering to the
executable space protection mechanisms inspired by "PaX". In addition, Syd
fortifies these MDWE protections by employing kernel-level seccomp filters on
critical system calls, including _mmap_(2), _mmap2_(2), _mprotect_(2),
_pkey_mprotect_(2), and _shmat_(2). These filters are designed to intercept and
restrict operations that could potentially contravene MDWE policies, such as
attempts to make non-executable memory mappings executable or to map shared
memory segments with executable permissions. By integrating "PR_SET_MDWE" for
preemptive kernel enforcement and utilizing seccomp filters for granular,
kernel-level control over system call execution, Syd provides a robust defence
mechanism against exploitation techniques that exploit memory vulnerabilities,
thereby ensuring a securely hardened execution environment. This restriction may
be relaxed using the "trace/allow_unsafe_memory:true" sandbox command at startup.
Note even with this restriction relaxed, Syd is going to call "PR_SET_MDWE", but
it will use the "PR_MDWE_NO_INHERIT" flag to prevent propagation of the MDWE
protection to child processes on _fork_(2).

As of version 3.25.0, Syd kills the process on memory errors rather than
denying these system calls with "EACCES". This ensures the system
administrator gets a notification via _dmesg_(1), and has a higher
chance to react soon to investigate potentially malicious activity. In
addition, repeated failures are going to trigger SegvGuard.

## Advanced Memory Protection Mechanisms

Syd version 3.15.1 enhances its security framework by integrating
sophisticated a seccomp BPF hook to meticulously block
_executable+shared_ memory mappings, targeting a critical vulnerability
exploitation pathway. As of version 3.21.3, Syd also blocks
_executable+anonymous_ memory. These updates refine the sandbox's
defence against unauthorised memory access and arbitrary code execution
by inspecting and filtering system calls, notably _mmap_(2), and
_mmap2_(2), to enforce stringent policies against dangerous memory
mapping combinations. While this bolstered security measure
significantly reduces the attack surface for exploits like buffer
overflows and code injections, it acknowledges potential legitimate use
cases, such as Just-In-Time (JIT) compilation and plugin architectures,
that may require exceptions. To accommodate necessary exceptions without
compromising overall security, Syd allows these restrictions to be
relaxed with explicit configuration through the
"trace/allow_unsafe_memory:true" command, ensuring that developers and
administrators can fine-tune the balance between security and
functionality according to specific requirements, with a keen eye on
preventing the propagation of relaxed security settings to child
processes.

## Null Address Mapping Prevention

In our ongoing effort to enhance the security features of Syd, as of
version 3.15.1 we introduced a crucial update inspired by the practices
of HardenedBSD, specifically aimed at bolstering our sandbox's defences
against null pointer dereference vulnerabilities. Following the model
set by HardenedBSD, Syd now includes a new security measure that
completely prohibits the mapping of memory at the NULL address using the
_mmap_(2) and _mmap2_(2) system calls with the MAP_FIXED and
MAP_FIXED_NOREPLACE flags. This addition is implemented through
meticulous seccomp filter rules that block these specific mapping
requests when the first argument (addr) is zero, effectively rendering
attempts to exploit null pointer dereferences as non-viable by ensuring
such memory allocations result in respective system call getting denied
with EACCES. By disallowing the execution of arbitrary code at the NULL
address, Syd significantly reduces the attack surface associated with
such vulnerabilities, reinforcing the sandbox's commitment to providing
a robust security framework for Linux systems. This technical
enhancement reflects our dedication to leveraging advanced security
insights from the broader community, embodying our proactive stance on
safeguarding against evolving threats.

Note, Linux has "vm/mmap_min_addr" which guards against this already.
Hence, this acts as a second layer of defense.

As of version 3.25.0, all adresses lower than the value of
"vm/mmap_min_addr" at Syd startup are included into the seccomp filter
the action of the filter is set to kill process rather than deny with
"EACCES". This ensures the system administrator gets a notification via
_dmesg_(1), and has a higher chance to react soon to investigate
potentially malicious activity. In addition, repeated failures are going
to trigger SegvGuard.

## Default Memory Allocator Security Enhancement

As of version 3.14.1, Syd has transitioned to using *mimalloc* with the _secure_
option as its default allocator. This enhancement introduces additional security
features such as guard pages, randomised allocation, and encrypted free lists.
While these measures significantly improve memory safety and hinder exploit
techniques that rely on predictable memory layouts or unprotected free lists,
they come with a performance overhead of approximately 10% as per *mimalloc*'s
own benchmarks. This strategic decision underscores Syd's commitment to
balancing performance with advanced security measures, offering users a more
secure sandboxing experience without compromising on efficiency.

## Enhanced Security for Memory File Descriptors

In version 3.21.1, Syd significantly enhanced its security posture by
introducing restrictions on memory file descriptors (memfds). The
_memfd_create_(2) system call is now sandboxed under Read, Write, and
Exec controls, with the name argument prepended with "/memfd:" before
access checks. This allows administrators to globally deny access to
memfds using rules like "deny/read,write,exec+/memfd:\*". Additionally,
the _memfd_secret_(2) system call, which requires the
"secretmem.enable=1" boot option and is seldom used, was denied to
prevent potential exploits. Despite file I/O being restricted on secret
memfds, they could be abused by attackers to write payloads and map them
as executable, thus bypassing denylisted code execution controls.

Building on these changes, version 3.21.2 further fortifies security by
making memfds non-executable by default. This is achieved by removing
the "MFD_EXEC" flag and adding the "MFD_NOEXEC_SEAL" flag to
_memfd_create_(2), ensuring memfds cannot be made executable. Notably,
the "MFD_NOEXEC_SEAL" flag requires Linux-6.3 or newer to function.
These measures collectively mitigate the risk of memfd abuse, which can
involve executing malicious code within a sandbox, circumventing
security mechanisms like Exec, Force, and TPE sandboxing. For scenarios
where executable or secret memfds are genuinely required, the
"trace/allow_unsafe_memfd:1" option allows for relaxing these
restrictions, though it introduces increased security risks. By default,
these enhancements enforce a robust security posture, preventing
attackers from leveraging memfds as a vector for unauthorized code
execution.

## Path Masking

Introduced in version 3.16.7, the "Path Masking" feature in Syd enhances
security by enabling the obfuscation of file contents without denying
access to the file itself. This functionality is critical in scenarios
where compatibility requires file presence, but not file readability.
Path Masking works by redirecting any attempt to _open_(2) a specified
file to the character device "/dev/null", effectively presenting an
empty file to the sandboxed process. The original file metadata remains
unchanged, which is essential for applications that perform operations
based on this data. Moreover, masked files can still be executed,
providing a seamless integration where executability is required but
content confidentiality must be preserved.

This feature leverages _glob_(3p) patterns to specify which files to
mask, allowing for flexible configuration tailored to diverse security
needs. By default, Syd masks sensitive paths such as "/proc/cmdline" to
prevent the leakage of potentially sensitive boot parameters, aligning
with Syd's security-first design philosophy. Path Masking is a robust
security enhancement that minimises the risk of sensitive data exposure
while maintaining necessary system functionality and compliance with
expected application behaviors.

## Refined Socket System Call Enforcement

In Syd version 3.16.12, we have strengthened the enforcement of socket system
call restrictions within the sandbox using kernel-level BPF filters. This
enhancement builds upon existing features by embedding these controls directly
into the Syd process, ensuring that even if Syd is compromised, it cannot
utilise or manipulate denied socket domains. This proactive measure restricts
socket creation strictly to permitted domains such as UNIX (AF_UNIX), IPv4
(AF_INET), and IPv6 (AF_INET6), significantly reducing the network attack
surface. The "trace/allow_unsupp_socket:1" option allows for the extension of
permissible socket domains, catering to specific needs but potentially
increasing exposure risks. Additionally, "trace/allow_safe_kcapi:1" enables access
to the Kernel Crypto API, facilitating necessary cryptographic operations
directly at the kernel level. These enhancements provide a more secure and
configurable environment, allowing administrators precise control over network
interactions and improving the overall security posture of the sandbox.

## Enhanced Execution Control (EEC)

The Enhanced Execution Control (EEC) feature, introduced in Syd version
3.17.0, represents a significant advancement in the sandbox’s defence
mechanisms. This feature strategically disables the _execve_(2) and
_execveat_(2) system calls for the Syd process after they are no longer
required for executing the sandbox process, thus safeguarding against
their potential abuse by a compromised Syd process. The prohibition of
these critical system calls adds a robust layer to the existing
Memory-Deny-Write-Execute (MDWE) protections, intensifying the system’s
defences against exploit techniques such as code injection or
return-oriented programming (ROP). Concurrently, EEC ensures that the
_ptrace_(2) syscall is limited following the initial use of the
"PTRACE_SEIZE" call for execution-related mitigations. This action
effectively prevents subsequent system trace operations, barring
unauthorised process attachments and further securing the system against
manipulation. Together, these measures enhance Syd's security
architecture, reflecting an ongoing commitment to implement rigorous,
state-of-the-art safeguards within the execution environment.

As of version 3.17.1, the Enhanced Execution Control (EEC) has been
further strengthened by integrating _mprotect_(2) hardening mechanisms
specifically targeting the prevention of the "ret2mprotect" exploitation
technique. This enhancement blocks attempts to alter memory protections
to executable (using the "PROT_EXEC" flag) via the _mprotect_(2) and
_pkey_mprotect_(2) system calls. By adding these checks, EEC mitigates the
risk associated with compromised Syd processes by enforcing stringent
memory operation policies that prevent unauthorised memory from becoming
executable, thereby countering sophisticated memory corruption attacks
such as return-oriented programming (ROP) and other code injection
strategies. This proactive security measure is crucial for maintaining
the integrity of the sandbox environment, ensuring that Syd continues to
offer robust protection against evolving exploit techniques.

As of version 3.23.9, the Enhanced Execution Control (EEC) feature has
been expanded to mitigate Sigreturn Oriented Programming (SROP) attacks
by denying access to the system calls _sigreturn_(2) and
_rt_sigreturn_(2) for _syd_(1), _syd-oci_(1), and _syd-tor_(1). Given
the lack of signal handlers, these system calls have no legitimate use.
By preventing these calls, the system is better protected against SROP
attacks, which involve manipulating signal handler frames to control
program state, thus significantly enhancing the security of the
execution environment.  For further reading, refer to section 2.4.4
Sigreturn-oriented programming in the Low-Level Software Security book
(URL:
https://llsoftsec.github.io/llsoftsecbook/#sigreturn-oriented-programming
). SROP (Bosman and Bos 2014) is a special case of ROP where the
attacker creates a fake signal handler frame and calls _sigreturn_(2), a
system call on many UNIX-type systems normally called upon return from a
signal handler, which restores the state of the process based on the
state saved on the signal handler’s stack by the kernel previously. The
ability to fake a signal handler frame and call sigreturn gives an
attacker a simple way to control the state of the program.

## Enhanced execve and execveat Syscall Validation

As of version 3.24.2, security enhancements to _execve_(2) and
_execveat_(2) syscalls have been introduced to thwart simple
Return-Oriented Programming (ROP) attacks. Per the Linux _execve_(2)
manpage: "On Linux, argv and envp can be specified as NULL. In both
cases, this has the same effect as specifying the argument as a pointer
to a list containing a single null pointer. *Do not take advantage of
this nonstandard and nonportable misfeature!* On many other UNIX
systems, specifying argv as NULL will result in an error (EFAULT). Some
other UNIX systems treat the envp==NULL case the same as Linux." Based
on this guidance, Syd now rejects _execve_(2) and _execveat_(2) with
"EFAULT" when one of the pathname, argv and envp arguments is NULL.
This mitigation targets basic ROP chains where NULL pointers are used as
placeholders to bypass argument validation checks, a common tactic in
exploiting buffer overflow vulnerabilities. For example, a typical ROP
chain trying to execute _execve_(2) with argv and envp set to NULL would
be intercepted and denied under these rules:

```
0x0000:         0x40ee2b pop rdx; ret
0x0008:              0x0 [arg2] rdx = 0
0x0010:         0x402885 pop rsi; ret
0x0018:              0x0 [arg1] rsi = 0
0x0020:         0x4013cc pop rdi; ret
0x0028:         0x460000 [arg0] rdi = 4587520
0x0030:         0x438780 execve
```

An attacker might circumvent this mitigation by ensuring that none of
the critical syscall arguments are NULL. This requires a more
sophisticated setup in the ROP chain, potentially increasing the
complexity of the exploit and reducing the number of vulnerable targets.
This focused security measure enhances system resilience against simple
ROP exploits while maintaining compliance with POSIX standards,
promoting robustness and cross-platform security.

As of version 3.25.0, Syd kills the process upon entering these system
calls with NULL arguments rather than denying them with "EFAULT". This
ensures the system administrator gets a notification via _dmesg_(1)
about potentially malicious activity. In addition, repeated failures are
going to trigger SegvGuard.

## Enhanced Path Integrity Measures

As of version 3.17.4, Syd incorporates crucial enhancements to maintain
the integrity of file system paths by systematically denying and masking
paths that contain control characters. These modifications are essential
for preventing the exploitation of terminal-based vulnerabilities and
for maintaining robustness in logging activities. Paths identified with
control characters are not only denied during sandbox access check but
are also sanitized when logged to ensure that potentially harmful data
does not compromise log integrity or facilitate inadvertent security
breaches. Such measures underscore Syd's ongoing commitment to
fortifying security by adhering to rigorous, up-to-date standards for
handling untrusted input efficiently.

As of version 3.18.6, this restriction can be relaxed by using the
setting "trace/allow_unsafe_filename:1"

## Device Sidechannel Mitigations

As of Syd version 3.21.0, Syd's device sidechannel mitigations align
closely with "GRKERNSEC_DEVICE_SIDECHANNEL" in Grsecurity, aiming to
prevent timing analyses on block or character devices via _stat_(2) or
_inotify_(7)/_fanotify_(7). For stat-related system calls, Syd, like
Grsecurity, matches the last access and modify times to the creation
time for devices, thwarting unprivileged user timing attacks. Instead of
dropping events, Syd strips access and modify _fanotify_(7)/_inotify_(7)
flags at syscall entry, preventing unsafe _fanotify_(7)/_inotify_(7)
event generation. This approach ensures unauthorized users cannot
determine sensitive information, such as the length of the administrator
password. Syd's solution offers robust security by dynamically stripping
flags, enhancing protection against these sidechannel attacks without
compromising functionality.

Refer to the following links for more information:

- https://web.archive.org/web/20130111093624/http://vladz.devzero.fr/013_ptmx-timing.php
- https://en.wikibooks.org/wiki/Grsecurity/Appendix/Grsecurity_and_PaX_Configuration_Options#Eliminate_stat/notify-based_device_sidechannels

## Restricting CPU Emulation System Calls

As of version 3.22.1, Syd denies the _modify_ldt_(2), _subpage_prot_(2),
_switch_endian_(2), _vm86_(2), and _vm86old_(2) system calls by default,
which are associated with CPU emulation functionalities. These calls can
only be allowed if the "trace/allow_unsafe_cpu" option is explicitly
set. This restriction helps mitigate potential vulnerabilities and
unauthorized access that can arise from modifying CPU state or memory
protections, thus strengthening the overall security posture of the
sandbox environment.

## Kernel Keyring Access Restriction

To enhance system security, access to the kernel's key management
facility via the _add_key_(2), _keyctl_(2), and _request_key_(2) system
calls is restricted by default as of version 3.22.1. These calls are
crucial for managing keys within the kernel, enabling operations such as
adding keys, manipulating keyrings, and requesting keys. The restriction
aims to prevent unauthorized or potentially harmful modifications to
keyrings, ensuring that only safe, controlled access is permitted.
However, administrators can relax this restriction by enabling the
"trace/allow_unsafe_keyring" option, allowing these system calls to be
executed when necessary for legitimate purposes.

Note, because of this restriction, Syd is not affected by
CVE-2024-42318 although we use Landlock. See here for more
information: https://www.openwall.com/lists/oss-security/2024/08/17/2

## Restricting Memory Protection Keys System Calls

As of version 3.22.1, Syd denies the system calls _pkey_alloc_(2),
_pkey_free_(2), and _pkey_mprotect_(2) by default. These system calls
are associated with managing memory protection keys, a feature that can
be leveraged to control memory access permissions dynamically. To allow
these system calls, administrators can enable the
"trace/allow_unsafe_pkey" option. This restriction enhances security by
preventing unauthorized or potentially harmful manipulations of memory
access permissions within the sandbox environment, ensuring stricter
control over memory protection mechanisms.

## Force Close-on-Exec

The "trace/force_cloexec" option, introduced in Syd version 0.22.1,
ensures that all _open_(2), _openat_(2), and _openat2_(2) system calls
made by the sandbox process include the "O_CLOEXEC" flag. This feature
can be toggled at runtime via Syd's virtual stat API, enabling dynamic
adjustment of confinement levels as needed. The "O_CLOEXEC" flag, when
set on file descriptors, ensures they are automatically closed when
executing a new program via _execve_(2) or similar system calls. This
automatic closure of file descriptors is critical for enhancing security
and safety, as it prevents file descriptors from being unintentionally
inherited by newly executed programs, which could otherwise lead to
unauthorized access to sensitive files or resources. By enforcing the
"O_CLOEXEC" flag across all _open_(2) calls, Syd mitigates the risk of
file descriptor leakage, effectively isolating the sandboxed environment
and ensuring a clean execution context for newly spawned processes. This
feature thus fortifies the security boundary of the sandbox, ensuring
that potentially sensitive file descriptors are not inadvertently
exposed to less privileged or potentially malicious code executed within
the sandbox.

## Restricting vmsplice System Call

As of version 3.23.5, Syd disables the _vmsplice_(2) system call by
default to enhance security. This syscall, identified as a potential
vector for memory corruption and privilege escalation, poses significant
risks in sandboxed environments. By default, disabling _vmsplice_(2)
reduces the attack surface, aligning with security practices in other
systems like Podman. Refer to the following links for more information:

- https://lore.kernel.org/linux-mm/X+PoXCizo392PBX7@redhat.com/
- https://lwn.net/Articles/268783/

## Enforcing Position-Independent Executables (PIE)

As of version 3.23.9, Syd mandates that all executables must be
Position-Independent Executables (PIE) to leverage Address Space Layout
Randomization (ASLR). PIE allows executables to be loaded at random
memory addresses, significantly enhancing security by making it more
difficult for attackers to predict the location of executable code. This
randomization thwarts various types of exploits, such as buffer overflow
attacks, which rely on predictable memory addresses to execute malicious
code. To accommodate scenarios where PIE is not feasible, administrators
can relax this restriction using the "trace/allow_unsafe_nopie:1" option.
This ensures compatibility while maintaining a robust security posture
by default, aligning with Syd's overarching strategy of employing
advanced security measures to mitigate potential attack vectors.

## Enforcing Non-Executable Stack

As of version 3.23.16, Syd mandates that all executables must have a
non-executable stack to enhance security. A non-executable stack helps
to prevent various types of exploits, such as stack-based buffer
overflow attacks, by making it more difficult for attackers to execute
malicious code from the stack. This security measure is similar to the
enforcement of Position-Independent Executables (PIE) and is a crucial
part of Syd's comprehensive security strategy. To accommodate scenarios
where a non-executable stack is not feasible, administrators can relax
this restriction using the "trace/allow_unsafe_stack:1" option. This
ensures compatibility while maintaining a robust security posture by
default, aligning with Syd's overarching strategy of employing advanced
security measures to mitigate potential attack vectors.

As of version 3.23.19, Syd enforces this restriction at _mmap_(2) boundary
as well so it is no longer possible to _dlopen_(3) a library with executable
stack to change the stack permissions of the process to executable. This
is useful in mitigating attacks such as CVE-2023-38408. Refer to the URL
https://www.qualys.com/2023/07/19/cve-2023-38408/rce-openssh-forwarded-ssh-agent.txt
for more information. As of version 3.25.0, Syd kills the process in
this case rather than denying the system call to be consistent with
other memory related seccomp filters. This ensures the system
administrator gets a notification via _syd-log_(1), and has a higher
chance to react soon to investigate potentially malicious activity. In
addition, repeated failures are going to trigger SegvGuard.

## Mitigation Against Heap Spraying

As of version 3.23.18, Syd introduces a critical security enhancement to
mitigate kernel heap-spraying attacks by restricting the _msgsnd_(2)
system call. This call, integral to System V message queues, is
essential for inter-process communication (IPC) in Unix-like operating
systems. System V message queues allow processes to send and receive
messages asynchronously, facilitating robust communication between
processes. However, it is also frequently exploited for heap spraying, a
technique that increases the predictability of memory allocations to
facilitate arbitrary code execution. Notably, exploits such as
CVE-2016-6187, CVE-2021-22555, and CVE-2021-26708 have leveraged this
system call for kernel heap-spraying to achieve privilege escalation and
kernel code execution. Heap spraying aims to introduce a high degree of
predictability to heap allocations, facilitating arbitrary code
execution by placing specific byte sequences at predictable memory
locations. This method is particularly dangerous because it increases
the reliability of exploiting vulnerabilities by aligning memory in a
way that malicious code execution becomes feasible. To counter this,
Syd now disables the _msgsnd_(2) system call by default, which is
commonly used for heap spraying due to its ability to allocate large,
contiguous blocks of memory in the kernel heap. This preemptive measure
significantly reduces the attack surface, preventing attackers from
leveraging this system call to bypass security mitigations and achieve
kernel code execution. Administrators can re-enable this call using the
"trace/allow_unsafe_msgsnd:1" option if required for legitimate
inter-process communication needs, ensuring that the default
configuration prioritizes security against such advanced exploitation
techniques.

For more information refer to the following links:
- https://en.wikipedia.org/wiki/Heap_spraying
- https://grsecurity.net/how_autoslab_changes_the_memory_unsafety_game
- https://duasynt.com/blog/cve-2016-6187-heap-off-by-one-exploit
- https://google.github.io/security-research/pocs/linux/cve-2021-22555/writeup.html
- https://a13xp0p0v.github.io/2021/02/09/CVE-2021-26708.html

## Mitigation against Page Cache Attacks

As of version 3.25.0, Syd denies the _mincore_(2) system call by default,
which is typically not needed during normal run and has been successfully
(ab)used for page cache attacks: https://arxiv.org/pdf/1901.01161

To quote the *Countermeasures* section of the article:

Our side-channel attack targets the operating system page cache via
operating system interfaces and behavior. Hence, it clearly can be
mitigated by modifying the operating system implementation. *Privileged
Access.* The _QueryWorkingSetEx_ and _mincore_ system calls are the core
of our side-channel attack. Requiring a higher privilege level for these
system calls stops our attack. The downside of restricting access to
these system calls is that existing programs which currently make use of
these system calls might break. Hence, we analyzed how frequently
_mincore_ is called by any of the software running on a typical Linux
installation. We used the Linux _perf_ tools to measure over a 5 hour
period whenever the _sys_enter_mincore_ system call is called by any
application. During these 5 hours a user performed regular operations on
the system, i.e., running various work-related tools like Libre Ofﬁce,
gcc, Clion, Thunderbird, Firefox, Nautilus, and Evince, but also
non-work-related tools like Spotify. The system was also running regular
background tasks during this time frame.  Surprisingly, the
_sys_enter_mincore_ system call was not called a single time. This
indicates that making the _mincore_ system call privileged is feasible
and would mitigate our attack at a very low implementation cost.

# HISTORY & DESIGN

- *sydbox-0* https://git.sr.ht/~alip/syd/tree/sydbox-0 is a _ptrace_(2) based sandbox.
- *sydbox-1* https://git.sr.ht/~alip/syd/tree/sydbox-1 is a _ptrace_(2) and _seccomp_(2) based sandbox.
- *sydbox-2* https://git.sr.ht/~alip/syd/tree/sydbox-1 is a _seccomp_(2) and _seccomp-notify_ based sandbox.
- *sydbox-3* is a rewrite of *sydbox-2* in Rust and it's what you are looking at.

This codebase has a history of a bit over 15 years and up to this point we have
used C11 as our implementation language for various reasons. With *sydbox-3*
we are moving forwards one step and writing the sandbox from scratch using the
Rust programming language with the only non-Rust dependency being libseccomp.
Although we inherit many ideas and design decisions from the old codebase, we
also don't shy away from radically changing the internal implementation making
it much simpler, idiomatic, and less prone to bugs. We have _proper multiarch
support_ since release 3.0.11, e.g on x86-64, you can run your x32 or x86
binaries just fine under Syd.

This version takes advantage of multithreading and handles system calls
using a thread pool whose size is equal to the number of CPUs on the
running machine and utilises globsets to match a list of patterns at
once, thus continues to perform reasonably well even with very long
rulesets. Moreover, access violations are logged into _syslog_(3), so
you may use a command like "journalctl SYSLOG_IDENTIFIER=syd" (or
shortly _syd-log_(1)) to view the sandbox logs. This version also comes
with four new sandboxing types called *Lock Sandboxing*, *Memory
Sandboxing*, *PID sandboxing*, *Stat Sandboxing*, *Force Sandboxing*:
*Lock Sandboxing* utilises the Landlock Linux Security Module (LSM),
*Memory Sandboxing* allows the user to define a per-process memory
limit, *PID sandboxing* allows the user to define a limit on the maximum
number of running tasks under the sandbox, *Stat Sandboxing* can be used
to effectively _hide files and directories_ from the sandboxed process
whereas *Force Sandboxing* can be used to verify file checksums prior to
exec, similar to HardenedBSD's Integriforce and NetBSD's Veriexec.

Finally, the new Syd has support for namespaces. Use e.g. "syd -m
unshare/user:1" to create a user namespace. You may use "mount", "uts", "ipc",
"pid", "net", and "cgroup" instead of "user" to create various namespaces. You
may use the "container" profile as a shorthand to create namespaces with
"syd -pcontainer".

You may use Syd as your login shell because it is very practical to have a
restricted user. To do this simply add "/path/to/syd" to the file "/etc/shells"
and do "chsh -s /path/to/syd username" as root. In this mode the sandbox may be
configured using the files "/etc/user.syd-3" and "~/.user.syd-3". If you want to
restrict user configuration of the sandbox, lock the sandbox using "lock:on" at
the end of the site-wide configuration file.

# EXHERBO

Syd is the default sandbox of *Exherbo Linux*. We use it to provide a restricted
environment under which package builds run with controlled access to file system
and network resources. *exheres-0* has a function called *esandbox* to interact
with Syd.

# BUGS

## TOCTOU due to pointer modification

In the operation of Syd, the only system calls that are not fully
emulated are _execve_(2) and _execveat_(2). As of version 3.16.0, Syd
applies an effective TOCTOU mitigation for these system calls utilizing
"PTRACE_EVENT_EXEC": To mitigate, Syd stops the process before return
from _exec_(3) using _ptrace_(2) and checks the updated "/proc/pid/exe"
magic symbolic link of the process against the sandbox. If the path is
found to be denylisted, which indicates a successful TOCTOU attempt to
execute a denylisted path, Syd kills the process. This mitigation may
be relaxed with the option "trace/allow_unsafe_exec:1", or with
"trace/allow_unsafe_debug:1" which implies the former.

Note, there is an open kernel bug to emulate _exec_(3) within seccomp
context: https://bugzilla.kernel.org/show_bug.cgi?id=218501

## Uninterruptible FIFOs

Due to the way seccomp-notify and signal handling interacts, FIFOs are
not interruptible under Syd. This means e.g. if an _open_(2) system call
is blocked for reading on a FIFO, sending the sandbox process that made
the original _open_(2) system call an interrupt signal will be queued by
the Linux kernel and _not_ delivered to the sandbox process until the
system call is handled by Syd. Therefore only opening the write end of
the FIFO can unblock the original blocked _open_(2) system call. This is
safe and secure, however it also breaks expectations of some programs
which rely on FIFOs being interruptible.

# SEE ALSO

_syd_(1), _syd_(2), _syd_(5), _seccomp_(2), _pidfd_getfd_(2), _pidfd_send
signal_(2), _ioctl_(2), _ioctl_tty_(2), _prctl_(2), _namespaces_(7),
_cgroup_namespaces_(7), _ipc_namespaces_(7), _mount_namespaces_(7),
_network_namespaces_(7), _pid_namespaces_(7), _user_namespaces_(7),
_uts_namespaces_(7)

- https://exherbolinux.org/docs/eapi/exheres-for-smarties.html#sandboxing

# AUTHORS

Maintained by Ali Polatel. Up-to-date sources can be found at
https://gitlab.exherbo.org/sydbox/sydbox.git and bugs/patches can be
submitted to https://gitlab.exherbo.org/groups/sydbox/-/issues. Discuss
in #sydbox on Libera Chat.
